@@ -1,32 +1,38 @@
-/*
- * The rsplib Prototype -- An RSerPool Implementation.
- * Copyright (C) 2005-2007 by Thomas Dreibholz, dreibh@exp-math.uni-essen.de
+/* $Id: asapinstance.h 2076 2008-12-28 15:52:34Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * Realized in co-operation between Siemens AG
- * and University of Essen, Institute of Computer Networking Technology.
+ *              //===//   //=====   //===//   //       //   //===//
+ *             //    //  //        //    //  //       //   //    //
+ *            //===//   //=====   //===//   //       //   //===<<
+ *           //   \\         //  //        //       //   //    //
+ *          //     \\  =====//  //        //=====  //   //===//    Version II
  *
- * Acknowledgement
- * This work was partially funded by the Bundesministerium fr Bildung und
- * Forschung (BMBF) of the Federal Republic of Germany (Förderkennzeichen 01AK045).
+ * ------------- An Efficient RSerPool Prototype Implementation -------------
+ *
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
+ *
+ * Acknowledgements:
+ * Realized in co-operation between Siemens AG and
+ * University of Essen, Institute of Computer Networking Technology.
+ * This work was partially funded by the Bundesministerium fuer Bildung und
+ * Forschung (BMBF) of the Federal Republic of Germany
+ * (Förderkennzeichen 01AK045).
  * The authors alone are responsible for the contents.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * Contact: rsplib-discussion@sctp.de
- *          dreibh@exp-math.uni-essen.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 #ifndef ASAPINSTANCE_H
@@ -70,8 +76,6 @@ struct ASAPInstance
    struct ST_CLASS(PoolHandlespaceManagement) Cache;
    struct ST_CLASS(PoolHandlespaceManagement) OwnPoolElements;
 
-   unsigned long long                         CacheElementTimeout;
-
    struct FDCallback                          RegistrarHuntFDCallback;
    struct FDCallback                          RegistrarFDCallback;
    struct Timer                               RegistrarTimeoutTimer;
@@ -82,8 +86,7 @@ struct ASAPInstance
 };
 
 
-#define ASAP_DEFAULT_CACHE_ELEMENT_TIMEOUT                     0
-#define ASAP_DEFAULT_REGISTRAR_ANNOUNCE_ADDRESS "239.0.0.1:3863"
+#define ASAP_DEFAULT_REGISTRAR_ANNOUNCE_ADDRESS ASAP_ANNOUNCE_MULTICAST_ADDRESS
 #define ASAP_DEFAULT_REGISTRAR_ANNOUNCE_TIMEOUT          5000000
 #define ASAP_DEFAULT_REGISTRAR_CONNECT_MAXTRIALS               3
 #define ASAP_DEFAULT_REGISTRAR_CONNECT_TIMEOUT           2500000
@@ -115,6 +118,7 @@ struct ASAPInstance* asapInstanceNew(struct Dispatcher*          dispatcher,
   */
 void asapInstanceDelete(struct ASAPInstance* asapInstance);
 
+
 /**
   * Register pool element.
   *
@@ -151,24 +155,31 @@ unsigned int asapInstanceDeregister(
   * @return RSPERR_OKAY in case of success; error code otherwise.
   */
 unsigned int asapInstanceReportFailure(struct ASAPInstance*            asapInstance,
-                               struct PoolHandle*              poolHandle,
-                               const PoolElementIdentifierType identifier);
+                                       struct PoolHandle*              poolHandle,
+                                       const PoolElementIdentifierType identifier);
 
 /**
-  * Do name resolution of given pool handle. The resulting pool pointer
-  * will be stored to the variable poolPtr.
+  * Do handle resolution of given pool handle. The result will contain
+  * an array of pointers to opaque data converted by the given conversion
+  * function from PoolElementNode structures. Conversion or at least copying
+  * is necessary, since other ASAP functions could modify the interal
+  * PoolElementNodes!
   *
   * @param asapInstance ASAPInstance.
   * @param poolHandle Pool handle.
-  * @param poolElementNodeArray Array to store pool element node pointers to.
-  * @param poolElementNodes Reference to variable containing maximum amount of pool element nodes to obtain. After function call, this variable contains actual amount of pool element nodes obtained.
+  * @param nodePtrArray Array to store pointers to converted PoolElementNodes to.
+  * @param nodePrts Reference to variable containing maximum amount of pool element nodes to obtain. After function call, this variable contains actual amount of pool element nodes obtained.
+  * @param cacheElementTimeout Stale cache value for newly received PE entries.
   * @return RSPERR_OKAY in case of success; error code otherwise.
   */
 unsigned int asapInstanceHandleResolution(
-                struct ASAPInstance*               asapInstanceInstance,
-                struct PoolHandle*                 poolHandle,
-                struct ST_CLASS(PoolElementNode)** poolElementNodeArray,
-                size_t*                            poolElementNodes);
+                struct ASAPInstance*     asapInstance,
+                struct PoolHandle*       poolHandle,
+                void**                   nodePtrArray,
+                size_t*                  nodePtrs,
+                unsigned int             (*convertFunction)(const struct ST_CLASS(PoolElementNode)* poolElementNode,
+                                                            void*                                   ptr),
+                const unsigned long long cacheElementTimeout);
 
 
 #ifdef __cplusplus
