@@ -1,4 +1,4 @@
-/* $Id: calcappclient.cc 1726 2008-02-14 09:56:55Z dreibh $
+/* $Id: calcappclient.cc 2076 2008-12-28 15:52:34Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2008 by Thomas Dreibholz
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,7 +41,12 @@
 #include "componentstatuspackets.h"
 #endif
 
+#include <stdio.h>
+#include <stdlib.h>
+#include <unistd.h>
+#include <string.h>
 #include <math.h>
+#include <signal.h>
 #include <ext_socket.h>
 #include <iostream>
 
@@ -629,7 +634,8 @@ void runProcess(const char* poolHandle, const char* objectName, unsigned long lo
                         (unsigned char*)poolHandle, strlen(poolHandle), 0) == 0) {
             sendCalcAppRequest(&process);
 
-            while((process.Status != PS_Failed) &&
+            while((!breakDetected()) &&
+                  (process.Status != PS_Failed) &&
                   (process.Status != PS_Completed)) {
 
                /* ====== Call rsp_poll() ================================= */
@@ -662,11 +668,6 @@ void runProcess(const char* poolHandle, const char* objectName, unsigned long lo
                   }
                   goto finished;
                }
-               if(breakDetected()) {
-                  rsp_close(process.RSerPoolSocketDescriptor);
-                  process.RSerPoolSocketDescriptor = -1;
-                  goto finished;
-               }
 
                if(process.Status == PS_Failover) {
                   handleFailover(&process);
@@ -678,6 +679,12 @@ void runProcess(const char* poolHandle, const char* objectName, unsigned long lo
                process.CurrentJob = process.Queue.dequeue();
                process.Status     = PS_Init;
             }
+
+            if(breakDetected()) {
+               rsp_close(process.RSerPoolSocketDescriptor);
+               process.RSerPoolSocketDescriptor = -1;
+               goto finished;
+            }
          }
          else {
             perror("rsp_connect() failed");
@@ -721,8 +728,10 @@ finished:
    fprintf(ScalarFH, "scalar \"%s\" \"CalcAppPU Total Startup Time\"         %1.6f\n", objectName, TotalStartupTime);
    fprintf(ScalarFH, "scalar \"%s\" \"CalcAppPU Total Processing Time\"      %1.6f\n", objectName, TotalProcessingTime);
    fprintf(ScalarFH, "scalar \"%s\" \"CalcAppPU Total Handling Time\"        %1.6f\n", objectName, TotalHandlingTime);
-   fprintf(ScalarFH, "scalar \"%s\" \"CalcAppPU Average Processing Speed\"   %1.0f\n", objectName, TotalJobSizeCompleted / TotalProcessingTime);
-   fprintf(ScalarFH, "scalar \"%s\" \"CalcAppPU Average Handling Speed\"     %1.0f\n", objectName, TotalJobSizeCompleted / TotalHandlingTime);
+   fprintf(ScalarFH, "scalar \"%s\" \"CalcAppPU Average Processing Speed\"   %1.0f\n", objectName,
+           (TotalProcessingTime > 0.0) ? TotalJobSizeCompleted / TotalProcessingTime : 0.0);
+   fprintf(ScalarFH, "scalar \"%s\" \"CalcAppPU Average Handling Speed\"     %1.0f\n", objectName,
+           (TotalHandlingTime > 0.0) ? TotalJobSizeCompleted / TotalHandlingTime : 0.0);
 
    fprintf(ScalarFH, "scalar \"%s\" \"CalcAppPU Total Jobs Queued\"          %llu\n",  objectName, TotalJobsQueued);
    fprintf(ScalarFH, "scalar \"%s\" \"CalcAppPU Total Jobs Started\"         %llu\n",  objectName, TotalJobsStarted);
@@ -749,6 +758,7 @@ finished:
 int main(int argc, char** argv)
 {
    struct rsp_info    info;
+   const char*        daemonPIDFile     = NULL;
    const char*        poolHandle        = "CalcAppPool";
    const char*        vectorFileName    = "CalcAppPU.vec";
    const char*        scalarFileName    = "CalcAppPU.sca";
@@ -783,13 +793,13 @@ int main(int argc, char** argv)
          JobSize = atof((char*)&argv[i][9]);
       }
       else if(!(strncmp(argv[i], "-keepalivetransmissioninterval=" ,31))) {
-         KeepAliveTransmissionInterval = atol((char*)&argv[i][31]);
+         KeepAliveTransmissionInterval = 1000ULL * atol((char*)&argv[i][31]);
          if(KeepAliveTransmissionInterval < 100000) {
             KeepAliveTransmissionInterval = 100000;
          }
       }
       else if(!(strncmp(argv[i], "-keepalivetimeoutinterval=" ,26))) {
-         KeepAliveTimeoutInterval = atol((char*)&argv[i][26]);
+         KeepAliveTimeoutInterval = 1000ULL * atol((char*)&argv[i][26]);
          if(KeepAliveTimeoutInterval < 100000) {
             KeepAliveTimeoutInterval = 100000;
          }
@@ -797,6 +807,9 @@ int main(int argc, char** argv)
       else if(!(strncmp(argv[i], "-runtime=" ,9))) {
          Runtime = (unsigned long long)rint(1000000.0 * atof((char*)&argv[i][9]));
       }
+      else if(!(strncmp(argv[i], "-daemonpidfile=", 15))) {
+         daemonPIDFile = (const char*)&argv[i][15];
+      }
       else if(!(strncmp(argv[i],"-log",4))) {
          if(initLogging(argv[i]) == false) {
             exit(1);
@@ -828,6 +841,23 @@ int main(int argc, char** argv)
         << endl;
 
 
+   if(daemonPIDFile != NULL) {
+      const pid_t childProcess = fork();
+      if(childProcess != 0) {
+         FILE* fh = fopen(daemonPIDFile, "w");
+         if(fh) {
+            fprintf(fh, "%d\n", childProcess);
+            fclose(fh);
+         }
+         else {
+            kill(childProcess, SIGKILL);
+            fprintf(stderr, "ERROR: Unable to create PID file \"%s\"!\n", daemonPIDFile);
+         }
+         exit(0);
+      }
+   }
+
+
    if(rsp_initialize(&info) < 0) {
       logerror("Unable to initialize rsplib");
       exit(1);
