@@ -1,37 +1,36 @@
-/*
- * The rsplib Prototype -- An RSerPool Implementation.
- * Copyright (C) 2005-2007 by Thomas Dreibholz, dreibh@exp-math.uni-essen.de
+/* $Id: componentstatuspackets.h 2076 2008-12-28 15:52:34Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: componentstatuspackets.h 1336 2007-02-13 14:08:22Z dreibh $
+ *              //===//   //=====   //===//   //       //   //===//
+ *             //    //  //        //    //  //       //   //    //
+ *            //===//   //=====   //===//   //       //   //===<<
+ *           //   \\         //  //        //       //   //    //
+ *          //     \\  =====//  //        //=====  //   //===//    Version II
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * Contact: rsplib-discussion@sctp.de
- *          dreibh@iem.uni-due.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 #ifndef COMPONENTSTATUSPACKETS_H
 #define COMPONENTSTATUSPACKETS_H
 
-#include <stdint.h>
-
-
-/* Ensure consistent alignment! */
-#pragma pack(push)
-#pragma pack(4)
+#include "tdtypes.h"
 
 
 #define CSP_VERSION        0x0200
@@ -57,7 +56,7 @@ struct ComponentStatusCommonHeader
    uint32_t Version;
    uint64_t SenderID;
    uint64_t SenderTimeStamp;
-};
+} __attribute__((packed));
 
 
 #define CSPR_LOCATION_SIZE 128
@@ -70,7 +69,7 @@ struct ComponentAssociation
    uint16_t Flags;
    uint16_t ProtocolID;
    uint32_t PPID;
-};
+} __attribute__((packed));
 
 struct ComponentStatusReport
 {
@@ -83,12 +82,9 @@ struct ComponentStatusReport
    uint16_t                           Workload;
    uint16_t                           Associations;
    struct ComponentAssociation        AssociationArray[0];
-};
+} __attribute__((packed));
 
 #define CSR_SET_WORKLOAD(w) ((w < 0.0) ? 0xffff : (uint16_t)rint(w * 0xfffe))
 #define CSR_GET_WORKLOAD(w) ((w == 0xffff) ? -1.0 : (double)(w / (double)0xfffe))
 
-
-#pragma pack(pop)
-
 #endif
