@@ -1,54 +1,72 @@
-/*
- * The rsplib Prototype -- An RSerPool Implementation.
- * Copyright (C) 2005-2007 by Thomas Dreibholz, dreibh@exp-math.uni-essen.de
+/* $Id: fractalpooluser.cc 2081 2008-12-28 20:16:37Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: fractalpooluser.cc 1336 2007-02-13 14:08:22Z dreibh $
+ *              //===//   //=====   //===//   //       //   //===//
+ *             //    //  //        //    //  //       //   //    //
+ *            //===//   //=====   //===//   //       //   //===<<
+ *           //   \\         //  //        //       //   //    //
+ *          //     \\  =====//  //        //=====  //   //===//    Version II
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * Contact: rsplib-discussion@sctp.de
- *          dreibh@iem.uni-due.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
-#include <qapplication.h>
-#include <qstatusbar.h>
-#include <qlayout.h>
-#include <qthread.h>
-#include <qpainter.h>
-#include <qcursor.h>
-#include <qfile.h>
-#include <qstringlist.h>
-#include <qdom.h>
+#include <QApplication>
+#include <QStatusBar>
+#include <QMenu>
+#include <QMessageBox>
+#include <QLayout>
+#include <QThread>
+#include <QTimer>
+#include <QPainter>
+#include <QCursor>
+#include <QFile>
+#include <QStringList>
+#include <QPaintEvent>
+#include <QDomDocument>
+#include <QMutex>
 #include <iostream>
 #include <complex>
 
+#include "fractalpooluser.h"
+
+#ifdef FRACTALPOOLUSER_USE_KDE
+#include <kcmdlineargs.h>
+#include <kstatusbar.h>
+#include <kaboutdata.h>
+#include <klocale.h>
+#endif
+
 #include "tdtypes.h"
 #include "rserpool.h"
-#include "fractalpooluser.h"
 #include "fractalgeneratorpackets.h"
 #include "timeutilities.h"
 #include "randomizer.h"
 #include "netutilities.h"
+#include "loglevel.h"
 #include "debug.h"
 #ifdef ENABLE_CSP
 #include "componentstatuspackets.h"
 #endif
 
+#include "fractalpooluser_moc.cc"
 
-using namespace std;
 
 
 #define DEFAULT_FPU_WIDTH             400
@@ -60,7 +78,9 @@ using namespace std;
 
 #define DEFAULT_FPU_CAPTION     NULL
 #define DEFAULT_FPU_POOLHANDLE  "FractalGeneratorPool"
+#ifndef DEFAULT_FPU_CONFIGDIR
 #define DEFAULT_FPU_CONFIGDIR   "fgpconfig"
+#endif
 #define DEFAULT_FPU_IMAGEPREFIX ""
 #define DEFAULT_FPU_COLORMARKS  TRUE
 
@@ -77,60 +97,47 @@ FractalPU::FractalPU(const size_t       width,
                      const char*        imageStoragePrefix,
                      const bool         colorMarks,
                      const size_t       threads,
-                     QWidget*           parent,
-                     const char*        name)
-   : QWidget(parent, name)
+                     QWidget*           parent)
+#ifdef FRACTALPOOLUSER_USE_KDE
+   : KMainWindow(parent)
+#else
+   : QMainWindow(parent)
+#endif
 {
-   QGridLayout* layout = new QGridLayout(this, 2, 1);
-   Q_CHECK_PTR(layout);
-   Display = new ImageDisplay(this, "Display");
+   Display = new ImageDisplay(this);
    Q_CHECK_PTR(Display);
-   StatusBar = new QStatusBar(this);
-   Q_CHECK_PTR(StatusBar);
    Display->setMinimumSize(width, height);
-   layout->addWidget(Display, 0, 0);
-   layout->addWidget(StatusBar, 1, 0);
-   layout->setRowStretch(0, 10);
-   layout->setRowStretch(1, 0);
-   PoolHandle     = (const unsigned char*)poolHandle;
-   PoolHandleSize = strlen((const char*)PoolHandle);
+   setCentralWidget(Display);
 
+   PoolHandle         = (const unsigned char*)poolHandle;
+   PoolHandleSize     = strlen((const char*)PoolHandle);
    RecvTimeout        = sendTimeout;
    SendTimeout        = recvTimeout;
    InterImageTime     = interImageTime;
    ImageStoragePrefix = QString(imageStoragePrefix);
    ColorMarks         = colorMarks;
-   Threads            = threads;
+   ConfiguredThreads  = threads;
    FileNumber         = 0;
 
    // ====== Initialize file and directory names ============================
-   ConfigDirectory = QDir(configDirName);
+   ConfigDirectory = QDir(configDirName, "*.fsf", QDir::Name, QDir::Files);
    if(ConfigDirectory.exists() != TRUE) {
-      cerr << "WARNING: Configuration directory " << configDirName << " does not exist!" << endl;
+      std::cerr << "WARNING: Configuration directory " << configDirName << " does not exist!" << std::endl;
       ConfigDirectory = QDir::current();
    }
-   const QString configFileName = ConfigDirectory.filePath("scenarios.conf");
-
-   // ====== Read configuration =============================================
-   QFile configFile(configFileName);
-   if(!configFile.open( IO_ReadOnly ) ) {
-      cerr << "WARNING: Unable to open configuration file "
-                << configFileName << "!" << endl;
-   }
    else {
-      QString buffer;
-      while(!configFile.atEnd()) {
-         configFile.readLine(buffer, 255);
-         buffer = buffer.stripWhiteSpace();
-         ConfigList.append(buffer);
+      QStringList contents = ConfigDirectory.entryList();
+      for(QStringList::Iterator iterator = contents.begin();iterator != contents.end();++iterator) {
+         // std::cout << *iterator << std::endl;
+         ConfigList.append(*iterator);
       }
    }
 
    // ====== Initialize widget and PU thread ================================
-   setCaption("Fractal Pool User");
-   StatusBar->message("Welcome to Fractal PU!", 3000);
+   setWindowTitle("Fractal Pool User");
+   statusBar()->showMessage("Welcome to Fractal PU!", 3000);
    show();
-   start();
+   QTimer::singleShot(100, this, SLOT(startNextJob()));
 }
 
 
@@ -138,89 +145,25 @@ FractalPU::FractalPU(const size_t       width,
 FractalPU::~FractalPU()
 {
    Status = FPU_Shutdown;
-   wait();
+   if(CalculationThreadArray) {
+      for(size_t i = 0;i < CurrentThreads;i++) {
+         CalculationThreadArray[i]->wait();
+         delete CalculationThreadArray[i];
+         CalculationThreadArray[i] = NULL;
+      }
+      delete [] CalculationThreadArray;
+      CalculationThreadArray = NULL;
+   }
    delete Display;
    Display = NULL;
-}
-
-
-/* ###### ImageDisplay constructor ####################################### */
-ImageDisplay::ImageDisplay(QWidget*    parent,
-                           const char* name)
-   : QWidget(parent, name)
-{
-   Image = NULL;
-}
-
-
-/* ###### ImageDisplay destructor ######################################## */
-ImageDisplay::~ImageDisplay()
-{
-   destroy();
-}
-
-
-/* ###### Initialize ImageDisplay ######################################## */
-void ImageDisplay::initialize(const size_t width, const size_t height)
-{
-   destroy();
-   if(Image == NULL) {
-      Image = new QImage(width, height, 32);
-      Q_CHECK_PTR(Image);
-      Image->fill(qRgb(200, 200, 200));
-   }
-}
-
-
-/* ###### Destroy ImageDisplay ########################################### */
-void ImageDisplay::destroy()
-{
-   if(Image) {
-      delete Image;
-      Image = NULL;
-   }
-}
-
-
-/* ###### Paint ImageDisplay ############################################# */
-void ImageDisplay::paintImage(const size_t startY, const size_t endY)
-{
-   if(Image) {
-      QPainter p;
-      p.begin(this);
-      p.drawImage(0, startY,
-                  *Image,
-                  0, startY,
-                  Image->width(), endY + 1 - startY);
-      p.end();
-   }
-}
-
-
-/* ###### Handle Paint event ############################################# */
-void ImageDisplay::paintEvent(QPaintEvent* paintEvent)
-{
-   qApp->lock();
-   if(Image) {
-      QPainter p;
-      p.begin(this);
-      p.drawImage(paintEvent->rect().left(), paintEvent->rect().top(),
-                  *Image,
-                  paintEvent->rect().left(),
-                  paintEvent->rect().top(),
-                  paintEvent->rect().width(),
-                  paintEvent->rect().height());
-      p.end();
-   }
-   qApp->unlock();
+   CurrentThreads = 0;
 }
 
 
 /* ###### Handle Close event ############################################# */
 void FractalPU::closeEvent(QCloseEvent* closeEvent)
 {
-   Status = FPU_Shutdown;
-   qApp->exit(0);
+   quit();
 }
 
 
@@ -242,10 +185,10 @@ void FractalPU::getNextParameters()
 
    QString parameterFileName(ConfigList[element]);
    QDomDocument doc("XMLFractalSave");
-   cout << "Getting parameters from " << ConfigDirectory.filePath(parameterFileName) << "..." << endl;
+   std::cout << "Getting parameters from " << (const char*)(ConfigDirectory.filePath(parameterFileName).toLocal8Bit().constData()) << "..." << std::endl;
    QFile parameterFile(ConfigDirectory.filePath(parameterFileName));
-   if(!parameterFile.open(IO_ReadOnly)) {
-      cerr << "WARNING: Cannot open parameter file " << parameterFileName << "!" << endl;
+   if(!parameterFile.open(QIODevice::ReadOnly)) {
+      std::cerr << "WARNING: Cannot open parameter file " << (const char*)(parameterFileName.toLocal8Bit().constData()) << "!" << std::endl;
       return;
    }
 
@@ -253,8 +196,8 @@ void FractalPU::getNextParameters()
    int line, column;
    if(!doc.setContent(&parameterFile, false, &error, &line, &column)) {
       parameterFile.close();
-      cerr << "WARNING: Error in parameter file " << parameterFileName << ":" << endl
-                << error << " in line " << line << ", column " << column << endl;
+      std::cerr << "WARNING: Error in parameter file " << (const char*)(parameterFileName.toLocal8Bit().constData()) << ":" << std::endl
+                << (const char*)(error.toLocal8Bit().constData()) << " in line " << line << ", column " << column << std::endl;
       return;
    }
    parameterFile.close();
@@ -268,8 +211,8 @@ void FractalPU::getNextParameters()
       Parameter.AlgorithmID = FGPA_MANDELBROT;
    }
    else {
-      cerr << "WARNING: Unknown algorithm name in parameter file " << parameterFileName << "!" << endl
-           << "Assuming Mandelbrot..." << endl;
+      std::cerr << "WARNING: Unknown algorithm name in parameter file " << (const char*)(parameterFileName.toLocal8Bit().constData()) << "!" << std::endl
+                << "Assuming Mandelbrot..." << std::endl;
       Parameter.AlgorithmID = FGPA_MANDELBROT;
    }
 
@@ -298,131 +241,268 @@ void FractalPU::getNextParameters()
 }
 
 
-/* ###### Fractal PU thread implementation ############################### */
-void FractalPU::run()
+/* ###### Start next fractal PU job ###################################### */
+void FractalPU::startNextJob()
 {
-   FractalCalculationThread* calculationThreadArray[Threads];
+   CHECK(Status != FPU_CalcInProgress);
 
-   Run    = 0;
-   while(Status != FPU_Shutdown) {
-      // ====== Initialize image object and timeout timer ===================
-      qApp->lock();
-      Status = FPU_CalcInProgress;
-      Run++;
-      Display->initialize(Display->width(), Display->height());
-      Parameter.Width         = Display->width();
-      Parameter.Height        = Display->height();
-      Parameter.C1Real        = -1.5;
-      Parameter.C1Imag        = 1.5;
-      Parameter.C2Real        = 1.5;
-      Parameter.C2Imag        = -1.5;
-      Parameter.N             = 2.0;
-      Parameter.MaxIterations = 1024;
-      Parameter.AlgorithmID   = FGPA_MANDELBROT;
-      getNextParameters();
-      qApp->unlock();
+   // ====== Initialize image object and timeout timer ======================
+   Run++;
+   Display->initialize(Display->width(), Display->height());
+   Parameter.Width         = Display->width();
+   Parameter.Height        = Display->height();
+   Parameter.C1Real        = -1.5;
+   Parameter.C1Imag        = 1.5;
+   Parameter.C2Real        = 1.5;
+   Parameter.C2Imag        = -1.5;
+   Parameter.N             = 2.0;
+   Parameter.MaxIterations = 1024;
+   Parameter.AlgorithmID   = FGPA_MANDELBROT;
+   getNextParameters();
 
 
-      // ====== Start job distribution ======================================
-      // cout << "Starting job distribution ..." << endl;
-      qApp->lock();
-      setCursor(Qt::WaitCursor);
-      StatusBar->message("Starting job distribution ...");
+   // ====== Start job distribution =========================================
+   Status = FPU_CalcInProgress;
 
-      const size_t yCount = (size_t)floor(sqrt((double)Threads));
-      const size_t yStep  = (size_t)rint((double)Parameter.Height / yCount);
-      size_t remaining = Threads;
-      size_t number    = 0;
-      for(size_t yPosition = 0;yPosition < yCount;yPosition++) {
-         const size_t xCount = (yPosition < yCount - 1) ? min(remaining, yCount) : remaining;
-         if(xCount > 0) {
-            const size_t xStep = (size_t)rint(Parameter.Width / xCount);
-            remaining -= xCount;
-            for(size_t xPosition = 0;xPosition < xCount;xPosition++) {
-               if((Threads > 1) && (ColorMarks)) {
-                  const QColor color(((5 * number) % 72) * 5, 100, 255, QColor::Hsv);
+   // std::cout << "Starting job distribution ..." << std::endl;
+   Display->setCursor(Qt::WaitCursor);
+   statusBar()->showMessage("Starting job distribution ...");
+
+   CurrentThreads = ConfiguredThreads;
+   CalculationThreadArray = new FractalCalculationThread*[CurrentThreads];
+   Q_CHECK_PTR(CalculationThreadArray);
+   for(size_t i = 0;i < CurrentThreads;i++) {
+      CalculationThreadArray[i] = NULL;
+   }
+
+
+   // ====== Initialize configured number of sessions =======================
+   QColor       color;
+   const size_t yCount = (size_t)floor(sqrt((double)CurrentThreads));
+   const size_t yStep  = (size_t)rint((double)Parameter.Height / yCount);
+
+   size_t remaining = CurrentThreads;
+   size_t number    = 0;
+   for(size_t yPosition = 0;yPosition < yCount;yPosition++) {
+      const size_t xCount = (yPosition < yCount - 1) ? min(remaining, yCount) : remaining;
+      if(xCount > 0) {
+         const size_t xStep = (size_t)rint(Parameter.Width / xCount);
+         remaining -= xCount;
+
+         // ====== Initialize rectange for next session =====================
+         const QPen numberPen(Qt::red);
+         QPainter   painter;
+         painter.begin(Display->Image);
+         painter.setPen(numberPen);
+         for(size_t xPosition = 0;xPosition < xCount;xPosition++) {
+            // ====== Mark rectange for next session ========================
+            if(CurrentThreads > 1) {
+               if(ColorMarks) {
+                  color.setHsv(((5 * number) % 72) * 5, 100, 255);
                   Display->fillRect(xPosition * xStep, yPosition * yStep,
                                     xStep, yStep, color.rgb());
                }
-
-               calculationThreadArray[number] =
-                  new FractalCalculationThread(this, number,
-                         xPosition * xStep, yPosition * yStep, xStep, yStep,
-                         (Threads == 1), ColorMarks);
-               Q_CHECK_PTR(calculationThreadArray[number]);
-               calculationThreadArray[number]->start();
-               number++;
+               char str[64];
+               snprintf((char*)&str, sizeof(str), "%u", (unsigned int)number + 1);
+               const QString numberString(str);
+               painter.drawText(xPosition * xStep, yPosition * yStep, xStep, yStep, Qt::AlignCenter, numberString);
             }
-         }
-         Display->paintImage(yPosition * yStep, (yPosition + 1) * yStep);
-      }
-      qApp->unlock();
-      CHECK(number == Threads);
 
-      // ====== Wait for job completion =====================================
-      // cout << "Waiting for job completion ..." << endl;
-      if(Threads > 1) {
-         qApp->lock();
-         StatusBar->message("Waiting for job completion ...");
-         qApp->unlock();
-      }
-      size_t failed = 0;
-      for(size_t i = 0;i < Threads;i++) {
-         calculationThreadArray[i]->wait();
-         if(!calculationThreadArray[i]->getSuccess()) {
-            failed++;
-         }
-      }
-      qApp->lock();
-      StatusBar->message((failed == 0) ? "Image completed!" : "Image calculation failed!");
-      setCursor(Qt::ArrowCursor);
-      for(size_t i = 0;i < Threads;i++) {
-         delete calculationThreadArray[i];   // Must be called with qApp locked!
-         calculationThreadArray[i] = NULL;
-      }
-      qApp->unlock();
+            // ====== Start new session =====================================
+            CalculationThreadArray[number] =
+               new FractalCalculationThread(this, number,
+                      xPosition * xStep, yPosition * yStep, xStep, yStep,
+                      (CurrentThreads == 1));
+            Q_CHECK_PTR(CalculationThreadArray[number]);
 
-      // ====== Save image ==================================================
-      if((failed == 0) && (ImageStoragePrefix != "")) {
-         QString fileName;
-         do {
-            FileNumber++;
-            QString fileSuffix;
-            fileSuffix.sprintf("-%06u.png", (unsigned int)FileNumber);
-            fileName = ImageStoragePrefix + fileSuffix;
-         } while((QFile::exists(fileName)) && (FileNumber < 999999));
-         StatusBar->message("Storing image as " + fileName);
-         if(!Display->Image->save(fileName, "PNG")) {
-            StatusBar->message("Storing image as " + fileName+ " failed!");
+            connect(CalculationThreadArray[number], SIGNAL(updateImage(int, int)),
+                    this, SLOT(redrawImage(int, int)));
+            connect(CalculationThreadArray[number], SIGNAL(updateStatus(QString)),
+                    this, SLOT(changeStatus(QString)));
+            connect(CalculationThreadArray[number], SIGNAL(finished()),
+                    this, SLOT(handleCompletedSession()));
+
+            CalculationThreadArray[number]->start();
+            number++;
          }
-         else {
-            StatusBar->message("Storing image as " + fileName+ " stored");
+         painter.end();
+      }
+   }
+   CHECK(number == CurrentThreads);
+
+   // ====== Make everything visible ========================================
+   Display->update();
+
+   // ====== Wait for job completion ========================================
+   // std::cout << "Waiting for job completion ..." << std::endl;
+   if(CurrentThreads > 1) {
+      statusBar()->showMessage("Waiting for job completion ...");
+   }
+}
+
+
+/* ###### Session has completed ########################################## */
+void FractalPU::handleCompletedSession()
+{
+   if(CalculationThreadArray) {
+      // ====== Count active sessions =======================================
+      size_t active = 0;
+      for(size_t i = 0;i < CurrentThreads;i++) {
+         if( (CalculationThreadArray[i] != NULL) && (!CalculationThreadArray[i]->isFinished()) ) {
+            active++;
          }
       }
 
-      // ====== Pause before next image =====================================
-      if(Status == FPU_CalcInProgress) {
-         char str[128];
-         size_t secsToWait = InterImageTime;
-         while(secsToWait > 0) {
-            usleep(1000000);
-            secsToWait--;
-            snprintf((char*)&str, sizeof(str), "Waiting for %u seconds ...", (int)secsToWait);
-            qApp->lock();
-            StatusBar->message(str);
-            qApp->unlock();
+      // ====== All sessions have finished ==================================
+      if(active == 0) {
+         Display->setCursor(Qt::ArrowCursor);
+
+         Success = true;
+         for(size_t i = 0;i < CurrentThreads;i++) {
+            CalculationThreadArray[i]->wait();
+            if(!CalculationThreadArray[i]->getSuccess()) {
+               Success = false;
+            }
+            delete CalculationThreadArray[i];
+            CalculationThreadArray[i] = NULL;
+         }
+         delete [] CalculationThreadArray;
+         CalculationThreadArray = NULL;
+
+         // ====== Pause before next image ==================================
+         if(Status == FPU_CalcInProgress) {
+            Status = FPU_Completed;
+            CountDown = InterImageTime;
+            QTimer::singleShot(0, this, SLOT(countDown()));
+         }
+         else if(Status == FPU_CalcAborted) {
+            Status = FPU_Completed;
+            statusBar()->showMessage("Restarting ...");
+            QTimer::singleShot(500, this, SLOT(startNextJob()));
          }
-      }
-      else if(Status == FPU_CalcAborted) {
-         qApp->lock();
-         StatusBar->message("Restarting ...");
-         qApp->unlock();
-         usleep(500000);
       }
    }
 }
 
 
+/* ###### Wait until start of next job ################################### */
+void FractalPU::countDown()
+{
+   char str[128];
+
+   snprintf((char*)&str, sizeof(str),
+            "%s - Waiting for %u seconds ...",
+            (Success == true) ? "Image completed" : "Image calculation failed",
+            CountDown);
+   statusBar()->showMessage(str);
+   if(CountDown > 0) {
+      CountDown--;
+      QTimer::singleShot(1000, this, SLOT(countDown()));
+   }
+   else {
+      QTimer::singleShot(1000, this, SLOT(startNextJob()));
+   }
+}
+
+
+/* ###### Redraw image ################################################### */
+void FractalPU::redrawImage(int start, int end)
+{
+   // Only update the lines which have changed.
+   Display->update(0, start, width(), end + 1 - start);
+}
+
+
+/* ###### Change text of status bar ###################################### */
+void FractalPU::changeStatus(QString statusText)
+{
+   statusBar()->showMessage(statusText);
+}
+
+
+/* ###### Context menu ################################################### */
+void FractalPU::contextMenuEvent(QContextMenuEvent* event)
+{
+   QMenu menu(this);
+
+   static unsigned int sessionsEntries[] = {1,
+                                            0,
+                                            2, 3, 4, 6, 8, 10,
+                                            0,
+                                            9, 16, 25, 36, 49, 64, 81, 100,
+                                            0,
+                                            144, 225, 256, 400};
+
+   QAction* showFailovers = menu.addAction("Show Failovers");
+   Q_CHECK_PTR(showFailovers);
+   showFailovers->setCheckable(true);
+   showFailovers->setChecked(ColorMarks);
+   connect(showFailovers, SIGNAL(toggled(bool)), this, SLOT(changeColorMarks(bool)));
+
+   QMenu* sessionsMenu = menu.addMenu("Sessions");
+   Q_CHECK_PTR(sessionsMenu);
+   for(size_t i = 0;i < sizeof(sessionsEntries) / sizeof(sessionsEntries[0]);i++) {
+      if(sessionsEntries[i] > 0) {
+         char str[64];
+         snprintf((char*)&str, sizeof(str), "%u", sessionsEntries[i]);
+         QAction* action = sessionsMenu->addAction(str);
+         Q_CHECK_PTR(action);
+         action->setCheckable(true);
+         if(sessionsEntries[i] == ConfiguredThreads) {
+            action->setChecked(true);
+         }
+         action->setData(sessionsEntries[i]);
+      }
+      else {
+         sessionsMenu->addSeparator();
+      }
+   }
+   connect(sessionsMenu, SIGNAL(triggered(QAction*)), this, SLOT(changeThreads(QAction*)));
+
+   menu.addSeparator();
+   menu.addAction("About", this, SLOT(about()));
+   menu.addSeparator();
+   menu.addAction("Quit", this, SLOT(quit()));
+
+   menu.exec(event->globalPos());
+}
+
+
+/* ###### Change ColorMarks setting ###################################### */
+void FractalPU::changeColorMarks(bool checked)
+{
+   ColorMarks = checked;
+}
+
+
+/* ###### Change Threads setting ######################################### */
+void FractalPU::changeThreads(QAction* action)
+{
+   const unsigned int newThreads = action->data().toInt();
+   if(newThreads >= 1) {
+      ConfiguredThreads = newThreads;
+      Status = FPU_CalcAborted;
+   }
+}
+
+
+/* ###### About ########################################################## */
+void FractalPU::about()
+{
+   QMessageBox::about(this, "About Fractal Pool User",
+      "<center><b>Fractal Pool User</b><br>Copyright (C) 2002-2009 by Thomas Dreibholz</center>");
+}
+
+
+/* ###### Quit ########################################################### */
+void FractalPU::quit()
+{
+   Status = FPU_Shutdown;
+   qApp->exit(0);
+}
+
+
+
 /* ###### Fractal PU thread constructor ################################## */
 FractalCalculationThread::FractalCalculationThread(FractalPU*         fractalPU,
                                                    const unsigned int threadID,
@@ -430,8 +510,7 @@ FractalCalculationThread::FractalCalculationThread(FractalPU*         fractalPU,
                                                    const size_t       viewY,
                                                    const size_t       viewWidth,
                                                    const size_t       viewHeight,
-                                                   const bool         showStatus,
-                                                   const bool         colorMarks)
+                                                   const bool         showStatus)
 
 {
    ThreadID   = threadID;
@@ -441,7 +520,6 @@ FractalCalculationThread::FractalCalculationThread(FractalPU*         fractalPU,
    ViewWidth  = viewWidth;
    ViewHeight = viewHeight;
    ShowStatus = showStatus;
-   ColorMarks = colorMarks;
    Session    = -1;
    Success    = false;
 }
@@ -472,12 +550,8 @@ bool FractalCalculationThread::sendParameterMessage()
    parameter.N             = doubleToNetwork(Master->Parameter.N);
 
    sent = rsp_sendmsg(Session, (char*)&parameter, sizeof(parameter), 0,
-                      0, htonl(PPID_FGP), 0, 0, Master->SendTimeout);
-   if(sent < 0) {
-      logerror("rsp_sendmsg() failed");
-      return(false);
-   }
-   return(true);
+                      0, htonl(PPID_FGP), 0, 0, 0, Master->SendTimeout);
+   return(sent > 0);
 }
 
 
@@ -493,7 +567,6 @@ FractalCalculationThread::DataStatus FractalCalculationThread::handleDataMessage
    size_t y      = ntohl(data->StartY);
    size_t points = ntohl(data->Points);
    if((points == 0) && (x == 0xffffffff) && (y == 0xffffffff)) {
-      // Master->update();
       return(Finalizer);
    }
    if(size < getFGPDataSize(points)) {
@@ -509,19 +582,22 @@ FractalCalculationThread::DataStatus FractalCalculationThread::handleDataMessage
       return(Invalid);
    }
 
+   Master->Display->lock();
+   QColor color;
    while(y < ViewHeight) {
       while(x < ViewWidth) {
          if(p >= points) {
             goto finished;
          }
          uint32_t point = ntohl(data->Buffer[p]);
-         if(ColorMarks) {
+         if(Master->ColorMarks) {
             point = (point + (2 * Master->Run) + (3 * ThreadID) + (5 * PoolElementUsages) % 72) * 5;
          }
          else {
             point = (point % 72) * 5;
          }
-         const QColor color(point, 255, 255, QColor::Hsv);
+         color.setHsv(point, 255, 255);
+         // color.setRgb(150,150,point % 255);
          Master->Display->setPixel(x + ViewX, y + ViewY, color.rgb());
          p++;
 
@@ -532,7 +608,8 @@ FractalCalculationThread::DataStatus FractalCalculationThread::handleDataMessage
    }
 
 finished:
-   Master->Display->paintImage(ntohl(data->StartY) + ViewY, y + ViewY);
+   Master->Display->unlock();
+   emit updateImage(ntohl(data->StartY) + ViewY, y + ViewY);
    return(Okay);
 }
 
@@ -546,17 +623,16 @@ void FractalCalculationThread::run()
 
    Session = rsp_socket(0, SOCK_SEQPACKET, IPPROTO_SCTP);
    if(Session >= 0) {
-      if(rsp_connect(Session, Master->PoolHandle, Master->PoolHandleSize) == 0) {
+      if(rsp_connect(Session, Master->PoolHandle, Master->PoolHandleSize, 0) == 0) {
 
          // ====== Begin image calculation ==================================
          do {
             if(ShowStatus) {
-               rsp_csp_setstatus(Session, 0, "Sending parameter command...");
-               // cout << "Sending parameter command..." << endl;
-               qApp->lock();
-               Master->StatusBar->message("Sending parameter command...");
-               qApp->unlock();
+               rsp_csp_setstatus(Session, 0, "Sending parameter command ...");
+               // cout << "Sending parameter command ..." << endl;
+               emit updateStatus("Sending parameter command ...");
             }
+
             if(sendParameterMessage()) {
                do {
 
@@ -592,9 +668,7 @@ void FractalCalculationThread::run()
                      else {
                         if((received >= (ssize_t)sizeof(FGPCommonHeader)) &&
                            (data.Header.Type == FGPT_DATA)) {
-                           qApp->lock();
                            const DataStatus status = handleDataMessage(&data, received);
-                           qApp->unlock();
 
                            switch(status) {
                               case Finalizer:
@@ -602,21 +676,19 @@ void FractalCalculationThread::run()
                                  goto finish;
                               break;
                               case Invalid:
-                                 cerr << "ERROR: Invalid data block received!" << endl;
+                                 std::cerr << "ERROR: Invalid data block received!" << std::endl;
                               break;
                               default:
                                  packets++;
                                  if(ShowStatus) {
                                     snprintf((char*)&statusText, sizeof(statusText),
-                                             "Processed packet #%u; PE is $%08x",
+                                             "Processed packet #%03u; PE is $%08x",
                                              (unsigned int)packets, rinfo.rinfo_pe_id);
                                     rsp_csp_setstatus(Session, 0, statusText);
                                     snprintf((char*)&statusText, sizeof(statusText),
-                                             "Processed data packet #%u (from PE $%08x)...",
+                                             "Processed packet #%03u from PE $%08x ...",
                                              (unsigned int)packets, rinfo.rinfo_pe_id);
-                                    qApp->lock();
-                                    Master->StatusBar->message(statusText);
-                                    qApp->unlock();
+                                    emit updateStatus(QString(statusText));
                                  }
                               break;
                            }
@@ -634,7 +706,7 @@ void FractalCalculationThread::run()
                   if(Success == false) {
                      printTimeStamp(stdout);
                      printf("FAILOVER (cookie=%s)...\n", (rsp_has_cookie(Session)) ? "yes" : "NO!");
-                     rsp_forcefailover(Session);
+                     rsp_forcefailover(Session, FFF_NONE, 0);
                   }
 
                } while(rsp_has_cookie(Session));
@@ -642,7 +714,7 @@ void FractalCalculationThread::run()
             else {
                printTimeStamp(stdout);
                printf("FAILOVER AFTER FAILED sendParameterMessage() (cookie=%s)...\n", (rsp_has_cookie(Session)) ? "yes" : "NO!");
-               rsp_forcefailover(Session);
+               rsp_forcefailover(Session, FFF_UNREACHABLE, 0);
             }
          } while(Success == false);
 
@@ -666,6 +738,63 @@ finish:
 
 
 
+/* ###### Imaay constructor ####################################### */
+ImageDisplay::ImageDisplay(QWidget* parent)
+   : QWidget(parent)
+{
+   Image = NULL;
+}
+
+
+/* ###### ImageDisplay destructor ######################################## */
+ImageDisplay::~ImageDisplay()
+{
+   destroy();
+}
+
+
+/* ###### Initialize ImageDisplay ######################################## */
+void ImageDisplay::initialize(const size_t width, const size_t height)
+{
+   destroy();
+   if(Image == NULL) {
+      Image = new QImage(width, height, QImage::Format_RGB32);
+      Q_CHECK_PTR(Image);
+      Image->fill(qRgb(200, 200, 200));
+   }
+}
+
+
+/* ###### Destroy ImageDisplay ########################################### */
+void ImageDisplay::destroy()
+{
+   if(Image) {
+      delete Image;
+      Image = NULL;
+   }
+}
+
+
+/* ###### Handle Paint event ############################################# */
+void ImageDisplay::paintEvent(QPaintEvent* paintEvent)
+{
+   if(Image) {
+      ImageMutex.lock();
+      QPainter p;
+      p.begin(this);
+      p.drawImage(paintEvent->rect().left(), paintEvent->rect().top(),
+                  *Image,
+                  paintEvent->rect().left(),
+                  paintEvent->rect().top(),
+                  paintEvent->rect().width(),
+                  paintEvent->rect().height());
+      p.end();
+      ImageMutex.unlock();
+   }
+}
+
+
+
 // ###### Main program ######################################################
 int main(int argc, char** argv)
 {
@@ -769,11 +898,12 @@ int main(int argc, char** argv)
    }
 
 
-   puts("Fractal Pool User - Version 1.0");
+   puts("Fractal Pool User - Version 2.0");
    puts("===============================\n");
    printf("Pool Handle          = %s\n", poolHandle);
    printf("Width                = %u\n", width);
    printf("Height               = %u\n", height);
+   printf("Config Directory     = %s\n", configDirName);
    printf("Send Timeout         = %u [ms]\n", sendTimeout);
    printf("Receive Timeout      = %u [ms]\n", recvTimeout);
    printf("Inter Image Time     = %u [s]\n", interImageTime);
@@ -788,13 +918,24 @@ int main(int argc, char** argv)
    }
 
 
+#ifdef FRACTALPOOLUSER_USE_KDE
+   KAboutData about("fractalpooluser", "fractalpooluser",
+                    ki18n("Fractal Pool User"),
+                    "2.0kde",
+                    ki18n("A KDE/RSerPool Fractal Generator Pool User"),
+                    KAboutData::License_GPL_V3,
+                    ki18n("Copyright (C) 2009 Thomas Dreibholz"));
+   KCmdLineArgs::init(argc, argv, &about);
+   KApplication application;
+#else
    QApplication application(argc, argv);
+#endif
    FractalPU* fractalPU = new FractalPU(width, height, poolHandle, configDirName,
                                         sendTimeout, recvTimeout, interImageTime,
                                         imageStoragePrefix, colorMarks, threads);
    Q_CHECK_PTR(fractalPU);
    if(caption) {
-      fractalPU->setCaption(caption);
+      fractalPU->setWindowTitle(caption);
    }
    fractalPU->show();
    const int result = application.exec();
