@@ -1,4 +1,4 @@
-/* $Id: netutilities.h 1744 2008-02-15 08:51:50Z dreibh $
+/* $Id: netutilities.h 2076 2008-12-28 15:52:34Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2008 by Thomas Dreibholz
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
  *
  * Acknowledgements:
  * Realized in co-operation between Siemens AG and
@@ -411,6 +411,7 @@ bool multicastGroupControl(int                         sd,
   * @param flags sendmsg() flags.
   * @param to Destination address or NULL for connection-oriented socket.
   * @param tolen Length of destination address or 0 if not given.
+  * @param ttl TTL for the multicast packet.
   */
 size_t sendmulticast(int                    sockfd,
                      int                    family,
@@ -418,7 +419,8 @@ size_t sendmulticast(int                    sockfd,
                      const size_t           length,
                      const int              flags,
                      const struct sockaddr* to,
-                     socklen_t              tolen);
+                     socklen_t              tolen,
+                     int                    ttl);
 
 /**
   * Set address and port reuse on socket on or off.
