@@ -1,4 +1,4 @@
-/* $Id: peerlistmanagement-template.h 1667 2008-02-04 10:12:05Z dreibh $
+/* $Id: peerlistmanagement-template.h 2086 2009-01-07 10:29:41Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //=====  //   //      //
@@ -9,7 +9,7 @@
  *
  * ------------- An Open Source RSerPool Simulation for OMNeT++ -------------
  *
- * Copyright (C) 2003-2008 by Thomas Dreibholz
+ * Copyright (C) 2003-2009 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -145,6 +145,9 @@ size_t ST_CLASS(peerListManagementPurgeExpiredPeerListNodes)(
 void ST_CLASS(peerListManagementVerifyChecksumsInHandlespace)(
         struct ST_CLASS(PeerListManagement)*        peerListManagement,
         struct ST_CLASS(PoolHandlespaceManagement)* poolHandlespaceManagement);
+struct ST_CLASS(PeerListNode)* ST_CLASS(peerListManagementGetUsefulPeerForPE)(
+                                  struct ST_CLASS(PeerListManagement)* peerListManagement,
+                                  const PoolElementIdentifierType      peIdentifier);
 
 
 #ifdef __cplusplus
