@@ -1,4 +1,4 @@
-/* $Id: registrar-enrp.c 1744 2008-02-15 08:51:50Z dreibh $
+/* $Id: registrar-enrp.c 2076 2008-12-28 15:52:34Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2008 by Thomas Dreibholz
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -205,8 +205,8 @@ void registrarHandlePeerEvent(struct Dispatcher* dispatcher,
 
 /* ###### Handle ENRP Handle Update ###################################### */
 void registrarHandleENRPHandleUpdate(struct Registrar*       registrar,
-                                     int                     fd,
-                                     sctp_assoc_t            assocID,
+                                     const int               fd,
+                                     const sctp_assoc_t      assocID,
                                      struct RSerPoolMessage* message)
 {
    struct ST_CLASS(PoolElementNode)* newPoolElementNode;
@@ -222,7 +222,9 @@ void registrarHandleENRPHandleUpdate(struct Registrar*       registrar,
       return;
    }
 
-   registrar->UpdateCount++;
+#ifdef ENABLE_REGISTRAR_STATISTICS
+   registrar->Stats.HandleUpdateCount++;
+#endif
 
    LOG_VERBOSE
    fputs("Got HandleUpdate for ", stdlog);
@@ -286,6 +288,19 @@ void registrarHandleENRPHandleUpdate(struct Registrar*       registrar,
             fputs("Handlespace content:\n", stdlog);
             registrarDumpHandlespace(registrar);
             LOG_END
+
+
+            /* ====== Takeover suggestion ================================ */
+            if( (message->Flags & EHF_TAKEOVER_SUGGESTED) &&
+                (registrar->ENRPSupportTakeoverSuggestion) &&
+                (newPoolElementNode->HomeRegistrarIdentifier != registrar->ServerID) ) {
+               LOG_ACTION
+               fprintf(stdlog, "Trying to take over PE $%08x from peer $%08x upon its suggestion\n",
+                       newPoolElementNode->Identifier, newPoolElementNode->HomeRegistrarIdentifier);
+               LOG_END
+               registrarSendASAPEndpointKeepAlive(registrar, newPoolElementNode, true);
+            }
+            /* =========================================================== */
          }
          else {
             LOG_WARNING
@@ -298,6 +313,11 @@ void registrarHandleENRPHandleUpdate(struct Registrar*       registrar,
             fputs("\n", stdlog);
             LOG_END
          }
+
+#ifdef ENABLE_REGISTRAR_STATISTICS
+         registrarWriteActionLog(registrar, "Recv", "ENRP", "Update", "AddPE", 0, 0, 0,
+                                 &message->Handle, message->PoolElementPtr->Identifier, message->SenderID, message->ReceiverID, 0, result);
+#endif
       }
       else {
          LOG_WARNING
@@ -312,7 +332,7 @@ void registrarHandleENRPHandleUpdate(struct Registrar*       registrar,
                   &registrar->Handlespace,
                   &message->Handle,
                   message->PoolElementPtr->Identifier);
-      if(message->Error == RSPERR_OKAY) {
+      if(result == RSPERR_OKAY) {
          LOG_ACTION
          fputs("Successfully deregistered ", stdlog);
          poolHandlePrint(&message->Handle, stdlog);
@@ -328,8 +348,11 @@ void registrarHandleENRPHandleUpdate(struct Registrar*       registrar,
          rserpoolErrorPrint(result, stdlog);
          fputs("\n", stdlog);
          LOG_END
-
       }
+#ifdef ENABLE_REGISTRAR_STATISTICS
+      registrarWriteActionLog(registrar, "Recv", "ENRP", "Update", "DelPE", 0, 0, 0,
+                              &message->Handle, message->PoolElementPtr->Identifier, message->SenderID, message->ReceiverID, 0, result);
+#endif
    }
 
    else {
@@ -349,6 +372,7 @@ void registrarSendENRPHandleUpdate(struct Registrar*                 registrar,
 #ifndef MSG_SEND_TO_ALL
    struct ST_CLASS(PeerListNode)* peerListNode;
 #endif
+   struct ST_CLASS(PeerListNode)* betterPeerListNode = NULL;
    struct RSerPoolMessage*        message;
 
    message = rserpoolMessageNew(NULL, 65536);
@@ -372,6 +396,25 @@ void registrarSendENRPHandleUpdate(struct Registrar*                 registrar,
       fputs("\n", stdlog);
       LOG_END
 
+      /* ====== Takeover suggestion ====================================== */
+      if( (action == PNUP_ADD_PE) &&
+          (registrar->ENRPSupportTakeoverSuggestion) &&
+          (poolElementNode->HomeRegistrarIdentifier == registrar->ServerID) ) {
+         betterPeerListNode = ST_CLASS(peerListManagementGetUsefulPeerForPE)(&registrar->Peers, poolElementNode->Identifier);
+         if(betterPeerListNode) {
+            LOG_ACTION
+            fprintf(stdlog, "Found better peer $%08x for PE $%08x\n",
+                    betterPeerListNode->Identifier, poolElementNode->Identifier);
+            LOG_END
+         }
+      }
+      /* ================================================================= */
+
+#ifdef ENABLE_REGISTRAR_STATISTICS
+      registrarWriteActionLog(registrar, "Send", "ENRP", "Update", ((message->Action == PNUP_ADD_PE) ? "AddPE" : "DelPE"), 0, 0, 0,
+                              &message->Handle, message->PoolElementPtr->Identifier, message->SenderID, message->ReceiverID, 0, 0);
+#endif
+
 #ifndef MSG_SEND_TO_ALL
       peerListNode = ST_CLASS(peerListManagementGetFirstPeerListNodeFromIndexStorage)(&registrar->Peers);
       while(peerListNode != NULL) {
@@ -397,6 +440,21 @@ void registrarSendENRPHandleUpdate(struct Registrar*                 registrar,
                           MSG_SEND_TO_ALL, 0, 0, message);
 #endif
 
+      if(betterPeerListNode) {
+         message->Flags |= EHF_TAKEOVER_SUGGESTED;
+         message->ReceiverID   = betterPeerListNode->Identifier;
+         message->AddressArray = betterPeerListNode->AddressBlock->AddressArray;
+         message->Addresses    = betterPeerListNode->AddressBlock->Addresses;
+         LOG_VERBOSE1
+         fprintf(stdlog, "Sending HandleUpdate to unicast peer $%08x with TakeoverSuggested flag...\n",
+                 betterPeerListNode->Identifier);
+         LOG_END
+         rserpoolMessageSend(IPPROTO_SCTP,
+                             registrar->ENRPUnicastSocket,
+                             0, 0, 0, 0,
+                             message);
+      }
+
       rserpoolMessageDelete(message);
    }
 }
@@ -404,8 +462,8 @@ void registrarSendENRPHandleUpdate(struct Registrar*                 registrar,
 
 /* ###### Handle ENRP List Request ####################################### */
 void registrarHandleENRPListRequest(struct Registrar*       registrar,
-                                    int                     fd,
-                                    sctp_assoc_t            assocID,
+                                    const int               fd,
+                                    const sctp_assoc_t      assocID,
                                     struct RSerPoolMessage* message)
 {
    struct RSerPoolMessage* response;
@@ -422,6 +480,10 @@ void registrarHandleENRPListRequest(struct Registrar*       registrar,
    fprintf(stdlog, "Got ListRequest from peer $%08x\n",
            message->SenderID);
    LOG_END
+#ifdef ENABLE_REGISTRAR_STATISTICS
+   registrarWriteActionLog(registrar, "Recv", "ENRP", "ListRequest", "", 0, 0, 0,
+                           NULL, 0, message->SenderID, message->ReceiverID, 0, 0);
+#endif
 
    response = rserpoolMessageNew(NULL, 65536);
    if(response != NULL) {
@@ -438,6 +500,11 @@ void registrarHandleENRPListRequest(struct Registrar*       registrar,
       fprintf(stdlog, "Sending ListResponse to peer $%08x...\n",
               message->SenderID);
       LOG_END
+#ifdef ENABLE_REGISTRAR_STATISTICS
+      registrarWriteActionLog(registrar, "Send", "ENRP", "ListResponse", "", message->Flags,
+                              ST_CLASS(peerListGetPeerListNodes)(&response->PeerListPtr->List), 0,
+                              NULL, 0, message->SenderID, message->ReceiverID, 0, 0);
+#endif
       if(rserpoolMessageSend(IPPROTO_SCTP,
                              fd, assocID, 0, 0, 0, response) == false) {
          LOG_WARNING
@@ -471,6 +538,10 @@ void registrarSendENRPListRequest(struct Registrar*           registrar,
       message->Flags        = 0x00;
       message->SenderID     = registrar->ServerID;
       message->ReceiverID   = receiverID;
+#ifdef ENABLE_REGISTRAR_STATISTICS
+      registrarWriteActionLog(registrar, "Send", "ENRP", "ListRequest", "", 0, 0, 0,
+                              NULL, 0, message->SenderID, message->ReceiverID, 0, 0);
+#endif
       if(rserpoolMessageSend(IPPROTO_SCTP,
                              sd, assocID, msgSendFlags, 0, 0, message) == false) {
          LOG_WARNING
@@ -504,6 +575,11 @@ void registrarHandleENRPListResponse(struct Registrar*       registrar,
    fprintf(stdlog, "Got ListResponse from peer $%08x\n",
            message->SenderID);
    LOG_END
+#ifdef ENABLE_REGISTRAR_STATISTICS
+   registrarWriteActionLog(registrar, "Recv", "ENRP", "ListResponse", "", message->Flags,
+                           (message->PeerListPtr != NULL) ? ST_CLASS(peerListGetPeerListNodes)(&message->PeerListPtr->List) : 0, 0,
+                           NULL, 0, message->SenderID, message->ReceiverID, 0, 0);
+#endif
 
    /* ====== Handle response ============================================= */
    if(!(message->Flags & EHF_LIST_RESPONSE_REJECT)) {
@@ -533,11 +609,11 @@ void registrarHandleENRPListResponse(struct Registrar*       registrar,
 
                   /* ====== New peer -> Send Peer Presence =============== */
                   registrarSendENRPPresence(registrar,
-                                   registrar->ENRPUnicastSocket, 0, 0,
-                                   newPeerListNode->AddressBlock->AddressArray,
-                                   newPeerListNode->AddressBlock->Addresses,
-                                   newPeerListNode->Identifier,
-                                   false);
+                                            registrar->ENRPUnicastSocket, 0, 0,
+                                            newPeerListNode->AddressBlock->AddressArray,
+                                            newPeerListNode->AddressBlock->Addresses,
+                                            newPeerListNode->Identifier,
+                                            false);
                }
             }
             else {
@@ -594,7 +670,11 @@ void registrarHandleENRPHandleTableRequest(struct Registrar*       registrar,
            message->SenderID);
    LOG_END
 
-   registrar->SynchronizationCount++;
+#ifdef ENABLE_REGISTRAR_STATISTICS
+   registrarWriteActionLog(registrar, "Recv", "ENRP", "HandleTableRequest", "", message->Flags, 0, 0,
+                           NULL, 0, message->SenderID, message->ReceiverID, 0, 0);
+   registrar->Stats.SynchronizationCount++;
+#endif
 
    peerListNode = ST_CLASS(peerListManagementFindPeerListNode)(
                      &registrar->Peers,
@@ -629,6 +709,11 @@ void registrarHandleENRPHandleTableRequest(struct Registrar*       registrar,
       fprintf(stdlog, "Sending HandleTableResponse to peer $%08x...\n",
               message->SenderID);
       LOG_END
+#ifdef ENABLE_REGISTRAR_STATISTICS
+      registrarWriteActionLog(registrar, "Send", "ENRP", "HandleTableResponse", "", message->Flags,
+                              (response->HandlespacePtr != NULL) ? ST_CLASS(poolHandlespaceManagementGetPoolElements)(response->HandlespacePtr) : 0, 0,
+                              NULL, 0, message->SenderID, message->ReceiverID, 0, 0);
+#endif
       if(rserpoolMessageSend(IPPROTO_SCTP,
                              fd, assocID, 0, 0, 0, response) == false) {
          LOG_WARNING
@@ -663,6 +748,10 @@ void registrarSendENRPHandleTableRequest(struct Registrar*           registrar,
       message->Flags        = flags;
       message->SenderID     = registrar->ServerID;
       message->ReceiverID   = receiverID;
+#ifdef ENABLE_REGISTRAR_STATISTICS
+      registrarWriteActionLog(registrar, "Send", "ENRP", "HandleTableRequest", "", message->Flags, 0, 0,
+                              NULL, 0, message->SenderID, message->ReceiverID, 0, 0);
+#endif
       if(rserpoolMessageSend(IPPROTO_SCTP,
                              sd, assocID, msgSendFlags, 0, 0, message) == false) {
          LOG_WARNING
@@ -700,6 +789,11 @@ void registrarHandleENRPHandleTableResponse(struct Registrar*       registrar,
    fprintf(stdlog, "Got HandleTableResponse from peer $%08x\n",
            message->SenderID);
    LOG_END
+#ifdef ENABLE_REGISTRAR_STATISTICS
+   registrarWriteActionLog(registrar, "Recv", "ENRP", "HandleTableResponse", "", message->Flags,
+                           (message->HandlespacePtr != NULL) ? ST_CLASS(poolHandlespaceManagementGetPoolElements)(message->HandlespacePtr) : 0, 0,
+                           NULL, 0, message->SenderID, message->ReceiverID, 0, 0);
+#endif
 
 
    /* ====== Find peer list node ========================================= */
@@ -872,8 +966,16 @@ void registrarHandleENRPPresence(struct Registrar*       registrar,
                                0,
                                &message->SourceAddress,
                                1, MAX_PE_TRANSPORTADDRESSES);
+#ifdef ENABLE_REGISTRAR_STATISTICS
+      registrarWriteActionLog(registrar, "Recv", "MulticastENRP", "Presence", "",  message->Flags, 0, 0,
+                              NULL, 0, message->SenderID, message->ReceiverID, 0, 0);
+#endif
    }
    else {
+#ifdef ENABLE_REGISTRAR_STATISTICS
+      registrarWriteActionLog(registrar, "Recv", "ENRP", "Presence", "", message->Flags, 0, 0,
+                              NULL, 0, message->SenderID, message->ReceiverID, 0, 0);
+#endif
       if(transportAddressBlockGetAddressesFromSCTPSocket(remoteAddressBlock,
                                                          fd, assocID,
                                                          MAX_PE_TRANSPORTADDRESSES,
@@ -923,14 +1025,16 @@ void registrarHandleENRPPresence(struct Registrar*       registrar,
       result = ST_CLASS(peerListManagementRegisterPeerListNode)(
                   &registrar->Peers,
                   message->PeerListNodePtr->Identifier,
-                  PLNF_DYNAMIC,
+                  PLNF_DYNAMIC|PLNF_NEW,   /* The entry is assumed to be new */
                   enrpTransportAddressBlock,
                   getMicroTime(),
                   &peerListNode);
 
       if(result == RSPERR_OKAY) {
          /* ====== Send Presence to new peer ============================= */
-         if(assocID == 0) {
+         /* PLNF_NEW will be removed when the entry was not new. If it is
+            still set, there is a new PR -> send him a Presence */
+         if(peerListNode->Flags & PLNF_NEW) {
             LOG_ACTION
             fputs("Sending Presence to new peer ", stdlog);
             ST_CLASS(peerListNodePrint)(peerListNode, stdlog, PLPO_FULL);
@@ -1112,6 +1216,10 @@ void registrarSendENRPPresence(struct Registrar*             registrar,
          messageLength = rserpoolMessage2Packet(message);
          if(messageLength > 0) {
             if(sd == registrar->ENRPMulticastOutputSocket) {
+#ifdef ENABLE_REGISTRAR_STATISTICS
+               registrarWriteActionLog(registrar, "Send", "MulticastENRP", "Presence", "", message->Flags, 0, 0,
+                                       NULL, 0, message->SenderID, message->ReceiverID, 0, 0);
+#endif
                if(sendmulticast(
                      registrar->ENRPMulticastOutputSocket,
                      registrar->ENRPMulticastOutputSocketFamily,
@@ -1119,13 +1227,18 @@ void registrarSendENRPPresence(struct Registrar*             registrar,
                      messageLength,
                      0,
                      &destinationAddressList->sa,
-                     getSocklen(&destinationAddressList->sa)) <= 0) {
+                     getSocklen(&destinationAddressList->sa),
+                     registrar->AnnounceTTL) <= 0) {
                   LOG_WARNING
                   fputs("Sending Presence via multicast failed\n", stdlog);
                   LOG_END
                }
             }
             else {
+#ifdef ENABLE_REGISTRAR_STATISTICS
+               registrarWriteActionLog(registrar, "Send", "ENRP", "Presence", "", message->Flags, 0, 0,
+                                       NULL, 0, message->SenderID, message->ReceiverID, 0, 0);
+#endif
                if(rserpoolMessageSend(IPPROTO_SCTP,
                                       sd, assocID, msgSendFlags, 0, 0, message) == false) {
                   LOG_WARNING
