@@ -1,4 +1,4 @@
-/* $Id: registrar.h 1816 2008-06-24 16:27:21Z dreibh $
+/* $Id: registrar.h 2076 2008-12-28 15:52:34Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2008 by Thomas Dreibholz
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -49,7 +49,10 @@
 #include <ext_socket.h>
 #include <net/if.h>
 #include <sys/ioctl.h>
+#define ENABLE_REGISTRAR_STATISTICS
+#ifdef ENABLE_REGISTRAR_STATISTICS
 #include <bzlib.h>
+#endif
 
 
 /* Exit immediately on Ctrl-C. No clean shutdown. */
@@ -74,6 +77,37 @@
 #define REGISTRAR_DEFAULT_TAKEOVER_EXPIRY_INTERVAL                    5000000
 #define REGISTRAR_DEFAULT_AUTOCLOSE_TIMEOUT                         300000000
 #define REGISTRAR_DEFAULT_ANNOUNCE_TTL                                     30
+#define REGISTRAR_DEFAULT_SUPPORT_TAKEOVER_SUGGESTION                   false
+#define REGISTRAR_DEFAULT_MAX_HR_RATE                                    -1.0   /* unlimited */
+#define REGISTRAR_DEFAULT_MAX_EU_RATE                                    -1.0   /* unlimited */
+
+
+#ifdef ENABLE_REGISTRAR_STATISTICS
+struct RegistrarStatistics
+{
+   unsigned long long                         StatsLine;
+   unsigned long long                         StatsStartTime;
+   unsigned long long                         ActionLogLine;
+   unsigned long long                         ActionLogLastActivity;
+   unsigned long long                         ActionLogStartTime;
+   int                                        StatsInterval;
+
+   unsigned long long                         RegistrationCount;
+   unsigned long long                         ReregistrationCount;
+   unsigned long long                         DeregistrationCount;
+   unsigned long long                         HandleResolutionCount;
+   unsigned long long                         FailureReportCount;
+   unsigned long long                         SynchronizationCount;
+   unsigned long long                         HandleUpdateCount;
+   unsigned long long                         EndpointKeepAliveCount;
+
+   bool                                       NeedsWeightedStatValues;
+   struct WeightedStatValue                   PoolsCount;
+   struct WeightedStatValue                   PoolElementsCount;
+   struct WeightedStatValue                   OwnedPoolElementsCount;
+   struct WeightedStatValue                   PeersCount;
+};
+#endif
 
 
 struct Registrar
@@ -85,7 +119,7 @@ struct Registrar
    struct ST_CLASS(PeerListManagement)        Peers;
    struct Timer                               HandlespaceActionTimer;
    struct Timer                               PeerActionTimer;
-
+   struct ST_CLASS(PoolUserList)              PoolUsers;
    struct MessageBuffer*                      UDPMessageBuffer;
 
    int                                        ASAPAnnounceSocket;
@@ -107,6 +141,7 @@ struct Registrar
    struct MessageBuffer*                      ENRPUnicastMessageBuffer;
    bool                                       ENRPAnnounceViaMulticast;
    struct Timer                               ENRPAnnounceTimer;
+   bool                                       ENRPSupportTakeoverSuggestion;
 
    bool                                       InStartupPhase;
    RegistrarIdentifierType                    MentorServerID;
@@ -128,31 +163,23 @@ struct Registrar
    unsigned long long                         PeerMaxTimeNoResponse;
    unsigned long long                         MentorDiscoveryTimeout;
    unsigned long long                         TakeoverExpiryInterval;
-
-   FILE*                                      ActionLogFile;
-   BZFILE*                                    ActionLogBZFile;
-   unsigned long long                         ActionLogLine;
-   unsigned long long                         ActionLogLastActivity;
-   unsigned long long                         ActionLogStartTime;
-   FILE*                                      StatsFile;
-   BZFILE*                                    StatsBZFile;
-   struct Timer                               StatsTimer;
-   unsigned long long                         StatsLine;
-   unsigned long long                         StatsStartTime;
-   int                                        StatsInterval;
-   unsigned long long                         RegistrationCount;
-   unsigned long long                         ReregistrationCount;
-   unsigned long long                         DeregistrationCount;
-   unsigned long long                         HandleResolutionCount;
-   unsigned long long                         FailureReportCount;
-   unsigned long long                         SynchronizationCount;
-   unsigned long long                         UpdateCount;
+   double                                     MaxHRRate;
+   double                                     MaxEURate;
 
 #ifdef ENABLE_CSP
    struct CSPReporter                         CSPReporter;
    unsigned int                               CSPReportInterval;
    union sockaddr_union                       CSPReportAddress;
 #endif
+
+#ifdef ENABLE_REGISTRAR_STATISTICS
+   struct RegistrarStatistics                 Stats;
+   FILE*                                      ActionLogFile;
+   BZFILE*                                    ActionLogBZFile;
+   FILE*                                      StatsFile;
+   BZFILE*                                    StatsBZFile;
+   struct Timer                               StatsTimer;
+#endif
 };
 
 
@@ -166,14 +193,17 @@ struct Registrar* registrarNew(const RegistrarIdentifierType  serverID,
                                const bool                     asapSendAnnounces,
                                const union sockaddr_union*    asapAnnounceAddress,
                                const bool                     enrpAnnounceViaMulticast,
-                               const union sockaddr_union*    enrpMulticastAddress,
-                               FILE*                          actionLogFile,
+                               const union sockaddr_union*    enrpMulticastAddress
+#ifdef ENABLE_REGISTRAR_STATISTICS
+                             , FILE*                          actionLogFile,
                                BZFILE*                        actionLogBZFile,
                                FILE*                          statsFile,
                                BZFILE*                        statsBZFile,
-                               unsigned int                   statsInterval
+                               const unsigned int             statsInterval,
+                               const bool                     needsWeightedStatValues
+#endif
 #ifdef ENABLE_CSP
-                               , const unsigned int           cspReportInterval,
+                             , const unsigned int             cspReportInterval,
                                const union sockaddr_union*    cspReportAddress
 #endif
                                );
@@ -187,15 +217,19 @@ unsigned long long registrarRandomizeCycle(const unsigned long long interval);
 unsigned int registrarRoundDistance(const unsigned int distance,
                                     const unsigned int step);
 void registrarUpdateDistance(struct Registrar*                       registrar,
-                             int                                     fd,
-                             sctp_assoc_t                            assocID,
+                             const int                               fd,
+                             const sctp_assoc_t                      assocID,
                              const struct ST_CLASS(PoolElementNode)* poolElementNode,
                              struct PoolPolicySettings*              updatedPolicySettings,
                              bool                                    addDistance,
                              unsigned int*                           distance);
 
 
-/* ###### Action Log ##################################################### */
+/* ###### Statistics ##################################################### */
+#ifdef ENABLE_REGISTRAR_STATISTICS
+void registrarPrintScalarStatistics(struct Registrar* registrar,
+                                    FILE*             fh,
+                                    const char*       objectName);
 void registrarBeginActionLog(struct Registrar* registrar);
 void registrarWriteActionLog(struct Registrar*         registrar,
                              const char*               direction,
@@ -211,6 +245,7 @@ void registrarWriteActionLog(struct Registrar*         registrar,
                              RegistrarIdentifierType   receiverID,
                              RegistrarIdentifierType   targetID,
                              unsigned int              errorCode);
+#endif
 
 
 /* ###### Core ########################################################### */
@@ -237,32 +272,32 @@ void registrarHandlePoolElementEvent(struct Dispatcher* dispatcher,
 
 /* ====== Handlespace Operations ======================= */
 void registrarRemovePoolElementsOfConnection(struct Registrar*  registrar,
-                                             const int          sd,
+                                             const int          fd,
                                              const sctp_assoc_t assocID);
 void registrarHandleASAPRegistration(struct Registrar*       registrar,
-                                     int                     fd,
-                                     sctp_assoc_t            assocID,
+                                     const int               fd,
+                                     const sctp_assoc_t      assocID,
                                      struct RSerPoolMessage* message);
 void registrarHandleASAPDeregistration(struct Registrar*       registrar,
-                                       int                     fd,
-                                       sctp_assoc_t            assocID,
+                                       const int               fd,
+                                       const sctp_assoc_t      assocID,
                                        struct RSerPoolMessage* message);
 void registrarHandleASAPHandleResolution(struct Registrar*       registrar,
-                                         int                     fd,
-                                         sctp_assoc_t            assocID,
+                                         const int               fd,
+                                         const sctp_assoc_t      assocID,
                                          struct RSerPoolMessage* message);
 
 /* ====== Monitoring =================================== */
 void registrarHandleASAPEndpointKeepAliveAck(struct Registrar*       registrar,
-                                             int                     fd,
-                                             sctp_assoc_t            assocID,
+                                             const int               fd,
+                                             const sctp_assoc_t      assocID,
                                              struct RSerPoolMessage* message);
 void registrarSendASAPEndpointKeepAlive(struct Registrar*                 registrar,
                                         struct ST_CLASS(PoolElementNode)* poolElementNode,
                                         const bool                        newHomeRegistrar);
 void registrarHandleASAPEndpointUnreachable(struct Registrar*       registrar,
-                                            int                     fd,
-                                            sctp_assoc_t            assocID,
+                                            const int               fd,
+                                            const sctp_assoc_t      assocID,
                                             struct RSerPoolMessage* message);
 
 
@@ -276,33 +311,33 @@ void registrarHandlePeerEvent(struct Dispatcher* dispatcher,
 
 /* ====== List/Handlespace I/O ========================= */
 void registrarHandleENRPHandleUpdate(struct Registrar*       registrar,
-                                     int                     fd,
-                                     sctp_assoc_t            assocID,
+                                     const int               fd,
+                                     const sctp_assoc_t      assocID,
                                      struct RSerPoolMessage* message);
 void registrarSendENRPHandleUpdate(struct Registrar*                 registrar,
                                    struct ST_CLASS(PoolElementNode)* poolElementNode,
                                    const uint16_t                    action);
 void registrarHandleENRPListRequest(struct Registrar*       registrar,
-                                    int                     fd,
-                                    sctp_assoc_t            assocID,
+                                    const int               fd,
+                                    const sctp_assoc_t      assocID,
                                     struct RSerPoolMessage* message);
 void registrarSendENRPListRequest(struct Registrar*           registrar,
-                                  int                         sd,
+                                  const int                   fd,
                                   const sctp_assoc_t          assocID,
                                   int                         msgSendFlags,
                                   const union sockaddr_union* destinationAddressList,
                                   const size_t                destinationAddresses,
                                   RegistrarIdentifierType     receiverID);
 void registrarHandleENRPListResponse(struct Registrar*       registrar,
-                                     int                     fd,
-                                     sctp_assoc_t            assocID,
+                                     const int               fd,
+                                     const sctp_assoc_t      assocID,
                                      struct RSerPoolMessage* message);
 void registrarHandleENRPHandleTableRequest(struct Registrar*       registrar,
-                                           int                     fd,
-                                           sctp_assoc_t            assocID,
+                                           const int               fd,
+                                           const sctp_assoc_t      assocID,
                                            struct RSerPoolMessage* message);
 void registrarSendENRPHandleTableRequest(struct Registrar*           registrar,
-                                         int                         sd,
+                                         const int                   fd,
                                          const sctp_assoc_t          assocID,
                                          int                         msgSendFlags,
                                          const union sockaddr_union* destinationAddressList,
@@ -310,17 +345,17 @@ void registrarSendENRPHandleTableRequest(struct Registrar*           registrar,
                                          RegistrarIdentifierType     receiverID,
                                          unsigned int                flags);
 void registrarHandleENRPHandleTableResponse(struct Registrar*       registrar,
-                                            int                     fd,
-                                            sctp_assoc_t            assocID,
+                                            const int               fd,
+                                            const sctp_assoc_t      assocID,
                                             struct RSerPoolMessage* message);
 
 /* ====== Handlespace Audit ============================ */
 void registrarHandleENRPPresence(struct Registrar*       registrar,
-                                 int                     fd,
+                                 const int               fd,
                                  sctp_assoc_t            assocID,
                                  struct RSerPoolMessage* message);
 void registrarSendENRPPresence(struct Registrar*             registrar,
-                               int                           sd,
+                               const int                     fd,
                                const sctp_assoc_t            assocID,
                                int                           msgSendFlags,
                                const union sockaddr_union*   destinationAddressList,
@@ -331,27 +366,34 @@ void registrarSendENRPPresenceToAllPeers(struct Registrar* registrar);
 
 /* ====== Takeover ===================================== */
 void registrarHandleENRPInitTakeover(struct Registrar*       registrar,
-                                     int                     fd,
+                                     const int               fd,
                                      sctp_assoc_t            assocID,
                                      struct RSerPoolMessage* message);
 void registrarSendENRPInitTakeoverToAllPeers(struct Registrar*             registrar,
                                              const RegistrarIdentifierType targetID);
 void registrarHandleENRPInitTakeoverAck(struct Registrar*       registrar,
-                                        int                     fd,
+                                        const int               fd,
                                         sctp_assoc_t            assocID,
                                         struct RSerPoolMessage* message);
 void registrarSendENRPInitTakeoverAck(struct Registrar*             registrar,
-                                      const int                     sd,
+                                      const int                     fd,
                                       const sctp_assoc_t            assocID,
                                       const RegistrarIdentifierType receiverID,
                                       const RegistrarIdentifierType targetID);
 
 void registrarHandleENRPTakeoverServer(struct Registrar*       registrar,
-                                       int                     fd,
+                                       const int               fd,
                                        sctp_assoc_t            assocID,
                                        struct RSerPoolMessage* message);
 void registrarSendENRPTakeoverServerToAllPeers(struct Registrar*             registrar,
                                                const RegistrarIdentifierType targetID);
 
+/* ###### Security ####################################################### */
+bool registrarPoolUserHasPermissionFor(struct Registrar*               registrar,
+                                       const int                       fd,
+                                       const sctp_assoc_t              assocID,
+                                       const unsigned int              action,
+                                       const struct PoolHandle*        poolHandle,
+                                       const PoolElementIdentifierType peIdentifier);
 
 #endif
