@@ -1,4 +1,4 @@
-/* $Id: rserpool.h 1732 2008-02-14 12:14:23Z dreibh $
+/* $Id: rserpool.h 2076 2008-12-28 15:52:34Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2008 by Thomas Dreibholz
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -167,16 +167,16 @@ struct rsp_addrinfo {
    uint32_t             ai_pe_id;
 };
 
-unsigned int rsp_pe_registration(const unsigned char* poolHandle,
-                                 const size_t         poolHandleSize,
-                                 struct rsp_addrinfo* rspAddrInfo,
-                                 struct rsp_loadinfo* rspLoadInfo,
-                                 unsigned int         registrationLife,
-                                 int                  flags);
+unsigned int rsp_pe_registration(const unsigned char*       poolHandle,
+                                 const size_t               poolHandleSize,
+                                 struct rsp_addrinfo*       rspAddrInfo,
+                                 const struct rsp_loadinfo* rspLoadInfo,
+                                 unsigned int               registrationLife,
+                                 const int                  flags);
 unsigned int rsp_pe_deregistration(const unsigned char* poolHandle,
                                    const size_t         poolHandleSize,
                                    const uint32_t       identifier,
-                                   int                  flags);
+                                   const int            flags);
 unsigned int rsp_pe_failure(const unsigned char* poolHandle,
                             const size_t         poolHandleSize,
                             const uint32_t       identifier);
