@@ -1,25 +1,30 @@
-/*
- * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004-2007 by Thomas Dreibholz
+/* $Id: rserpoolerror.c 2086 2009-01-07 10:29:41Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: rserpoolerror.c 1286 2007-01-11 08:24:02Z dreibh $
+ *              //===//   //=====   //===//   //=====  //   //      //
+ *             //    //  //        //    //  //       //   //=/  /=//
+ *            //===//   //=====   //===//   //====   //   //  //  //
+ *           //   \\         //  //             //  //   //  //  //
+ *          //     \\  =====//  //        =====//  //   //      //  Version V
  *
- * This program is free software; you can redistribute it and/or modify
+ * ------------- An Open Source RSerPool Simulation for OMNeT++ -------------
+ *
+ * Copyright (C) 2003-2009 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * Contact: dreibh@exp-math.uni-essen.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 #include "rserpoolerror.h"
@@ -40,8 +45,9 @@ static struct ErrorTable ErrorDescriptions[] = {
    /* Protocol-specific error causes */
    { RSPERR_UNRECOGNIZED_PARAMETER,        "unrecognized parameter" },
    { RSPERR_UNRECOGNIZED_MESSAGE,          "unrecognized message" },
-   { RSPERR_AUTHORIZATION_FAILURE,         "authorization failure" },
-   { RSPERR_INVALID_VALUES,                "invalid values" },
+   { RSPERR_INVALID_MESSAGE,               "invalid message" },
+   { RSPERR_INVALID_TLV,                   "invalid TLV" },
+   { RSPERR_INVALID_VALUE,                 "invalid value" },
    { RSPERR_NO_USABLE_USER_ADDRESSES,      "no usable user endpoint address(es)" },
    { RSPERR_NO_USABLE_ASAP_ADDRESSES,      "no usable ASAP endpoint address(es)" },
 
@@ -57,7 +63,7 @@ static struct ErrorTable ErrorDescriptions[] = {
    { RSPERR_UNRECOGNIZED_PARAMETER_SILENT, "unrecognized parameter (silent)" },
 
    /* Handlespace-management specific error causes */
-   { RSPERR_NO_RESOURCES,                  "no resources" },
+   { RSPERR_OUT_OF_RESOURCES,              "out of resources" },
    { RSPERR_NOT_FOUND,                     "object not found" },
    { RSPERR_INVALID_ID,                    "invalid ID" },
    { RSPERR_OWN_ID,                        "own ID" },
