@@ -1,4 +1,4 @@
-/* $Id: scriptingservice.h 1684 2008-02-06 11:47:08Z dreibh $
+/* $Id: scriptingservice.h 2076 2008-12-28 15:52:34Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2008 by Thomas Dreibholz
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -51,7 +51,7 @@ class ScriptingServer : public TCPLikeServer
                    ScriptingServerSettings* settings);
    ~ScriptingServer();
 
-   static TCPLikeServer* scriptingServerFactory(int sd, void* userData);
+   static TCPLikeServer* scriptingServerFactory(int sd, void* userData, const uint32_t peIdentifier);
    static void scriptingPrintParameters(const void* userData);
 
    protected:
