@@ -1,4 +1,4 @@
-/* $Id: server.cc 1793 2008-06-08 16:50:59Z dreibh $
+/* $Id: server.cc 2076 2008-12-28 15:52:34Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2008 by Thomas Dreibholz
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include "breakdetector.h"
 #include "tagitem.h"
 #include "netutilities.h"
+#include "randomizer.h"
 #include "standardservices.h"
 #include "fractalgeneratorservice.h"
 #include "calcappservice.h"
@@ -44,7 +45,9 @@
 #include <unistd.h>
 #include <string.h>
 #include <math.h>
-
+#include <sys/types.h>
+#include <signal.h>
+      
 
 #define SERVICE_ECHO      1
 #define SERVICE_DISCARD   2
@@ -68,9 +71,13 @@ int main(int argc, char** argv)
    unsigned int        runtimeLimit  = 0;
    unsigned int        service       = SERVICE_ECHO;
    const char*         poolHandle    = NULL;
+   const char*         daemonPIDFile = NULL;
    bool                policyChanged = false;
    bool                quiet         = false;
+   double              uptime        = 0.0;
+   double              downtime      = 0.0;
 
+start:
    /* ====== Read parameters ============================================= */
    rsp_initinfo(&info);
    tags[0].Tag  = TAG_PoolElement_Identifier;
@@ -98,14 +105,17 @@ int main(int argc, char** argv)
 #endif
       }
       else if(!(strncmp(argv[i], "-poolhandle=" ,12))) {
-         poolHandle = (char*)&argv[i][12];
+         poolHandle = (const char*)&argv[i][12];
       }
       else if(!(strncmp(argv[i], "-rereginterval=" ,15))) {
-         reregInterval = atol((char*)&argv[i][15]);
+         reregInterval = atol((const char*)&argv[i][15]);
          if(reregInterval < 10) {
             reregInterval = 10;
          }
       }
+      else if(!(strncmp(argv[i], "-daemonpidfile=", 15))) {
+         daemonPIDFile = (const char*)&argv[i][15];
+      }
       else if(!(strncmp(argv[i], "-policy=" ,8))) {
          double dpf;
 
@@ -226,11 +236,24 @@ int main(int argc, char** argv)
       else if(!(strcmp(argv[i], "-quiet"))) {
          quiet = true;
       }
+      else if(!(strncmp(argv[i], "-uptime=", 8))) {
+         uptime = atof((const char*)&argv[i][8]);
+      }
+      else if(!(strncmp(argv[i], "-downtime=", 10))) {
+         downtime = atof((const char*)&argv[i][10]);
+      }
+   }
+   if(uptime > 0.000001) {
+      runtimeLimit = (unsigned long long)rint(1000.0 * randomExpDouble(uptime));
    }
 
 
    /* ====== Print startup message ======================================= */
    if(!quiet) {
+      if(uptime > 0.000001) {
+         printf("Uptime   = %1.3f [s]\n", uptime);
+         printf("Downtime = %1.3f [s]\n", downtime);
+      }
       printf("Starting service ");
       if(service == SERVICE_ECHO) {
          printf("Echo");
@@ -248,6 +271,23 @@ int main(int argc, char** argv)
    }
 
 
+   if(daemonPIDFile != NULL) {
+      const pid_t childProcess = fork();
+      if(childProcess != 0) {
+         FILE* fh = fopen(daemonPIDFile, "w");
+         if(fh) {
+            fprintf(fh, "%d\n", childProcess);
+            fclose(fh);
+         }
+         else {
+            kill(childProcess, SIGKILL);
+            fprintf(stderr, "ERROR: Unable to create PID file \"%s\"!\n", daemonPIDFile);
+         }
+         exit(0);
+      }
+   }
+
+
    /* ====== Start requested service ===================================== */
    if(service == SERVICE_ECHO) {
       EchoServer echoServer;
@@ -367,6 +407,8 @@ int main(int argc, char** argv)
                                  (struct TagItem*)&tags);
    }
    else if(service == SERVICE_CALCAPP) {
+      static CalcAppServer::CalcAppServerStatistics stats;
+      static bool        resetStatistics               = true;
       const char*        objectName                    = "scenario.calcapppoolelement[0]";
       const char*        vectorFileName                = "calcapppoolelement.vec";
       const char*        scalarFileName                = "calcapppoolelement.sca";
@@ -375,6 +417,7 @@ int main(int argc, char** argv)
       unsigned long long keepAliveTimeoutInterval      = 2500000;
       unsigned long long cookieMaxTime                 = 5000000;
       double             cookieMaxCalculations         = 5000000.0;
+      double             cleanShutdownProbability      = 1.0;
       size_t             maxJobs                       = 10;
       for(int i = 1;i < argc;i++) {
          if(!(strncmp(argv[i], "-capobject=" ,11))) {
@@ -387,52 +430,63 @@ int main(int argc, char** argv)
             scalarFileName = (const char*)&argv[i][11];
          }
          else if(!(strncmp(argv[i], "-capmaxjobs=" ,12))) {
-            maxJobs = atol((char*)&argv[i][12]);
+            maxJobs = atol((const char*)&argv[i][12]);
             if(maxJobs < 1) {
                maxJobs = 1;
             }
          }
          else if(!(strncmp(argv[i], "-capkeepalivetransmissioninterval=" ,34))) {
-            keepAliveTransmissionInterval = atol((char*)&argv[i][34]);
+            keepAliveTransmissionInterval = 1000ULL * atol((const char*)&argv[i][34]);
             if(keepAliveTransmissionInterval < 100000) {
                keepAliveTransmissionInterval = 100000;
             }
          }
          else if(!(strncmp(argv[i], "-capkeepalivetimeoutinterval=" ,29))) {
-            keepAliveTimeoutInterval = atol((char*)&argv[i][29]);
+            keepAliveTimeoutInterval = 1000ULL * atol((const char*)&argv[i][29]);
             if(keepAliveTimeoutInterval < 100000) {
                keepAliveTimeoutInterval = 100000;
             }
          }
          else if(!(strncmp(argv[i], "-capcapacity=" ,13))) {
-            capacity = atof((char*)&argv[i][13]);
+            capacity = atof((const char*)&argv[i][13]);
             if(capacity < 1.0) {
                capacity = 1.0;
             }
          }
          else if(!(strncmp(argv[i], "-capcookiemaxcalculations=" ,26))) {
-            cookieMaxCalculations = atof((char*)&argv[i][26]);
+            cookieMaxCalculations = atof((const char*)&argv[i][26]);
             if(cookieMaxCalculations < 1.0) {
                cookieMaxCalculations = 1.0;
             }
          }
          else if(!(strncmp(argv[i], "-capcookiemaxtime=" ,18))) {
-            cookieMaxTime = atol((char*)&argv[i][18]);
+            cookieMaxTime = (unsigned long long)rint(1000000.0 * atof((const char*)&argv[i][18]));
             if(cookieMaxTime < 100000) {
                cookieMaxTime = 100000;
             }
          }
+         else if(!(strncmp(argv[i], "-capcleanshutdownprobability=" , 29))) {
+            cleanShutdownProbability = atof((const char*)&argv[i][29]);
+            if(cleanShutdownProbability < 0.0) {
+               cleanShutdownProbability = 0.0;
+            }
+            else if(cleanShutdownProbability > 1.0) {
+               cleanShutdownProbability = 1.0;
+            }
+         }
       }
 
       CalcAppServer calcAppServer(maxJobs, objectName, vectorFileName, scalarFileName,
                                   capacity,
                                   keepAliveTransmissionInterval, keepAliveTimeoutInterval,
-                                  cookieMaxTime, cookieMaxCalculations);
+                                  cookieMaxTime, cookieMaxCalculations, cleanShutdownProbability,
+                                  &stats, resetStatistics);
       calcAppServer.poolElement("CalcApp Server - Version 1.0",
                                 (poolHandle != NULL) ? poolHandle : "CalcAppPool",
                                 &info, &loadInfo,
                                 reregInterval, runtimeLimit, quiet,
                                 (struct TagItem*)&tags);
+      resetStatistics = false;
    }
    else if(service == SERVICE_SCRIPTING) {
       size_t maxThreads = 1;
@@ -471,5 +525,15 @@ int main(int argc, char** argv)
    }
 
    rsp_freeinfo(&info);
+
+   // ====== Restart ========================================================
+   if((uptime > 0.000001) && (!breakDetected())) {
+      const double t = randomExpDouble(downtime);
+      printf("\nWaiting for %1.3fs ...\n", t);
+      usleep((unsigned long long)rint(t * 1000000.0));
+      if(!breakDetected()) {
+         goto start;
+      }
+   }
    return(0);
 }
