@@ -1,25 +1,30 @@
-/*
- * An Efficient RSerPool Pool Handlespace Management Implementation
- * Copyright (C) 2004-2007 by Thomas Dreibholz
+/* $Id: transportaddressblock.c 2086 2009-01-07 10:29:41Z dreibh $
+ * --------------------------------------------------------------------------
  *
- * $Id: transportaddressblock.c 1361 2007-05-09 13:55:39Z dreibh $
+ *              //===//   //=====   //===//   //=====  //   //      //
+ *             //    //  //        //    //  //       //   //=/  /=//
+ *            //===//   //=====   //===//   //====   //   //  //  //
+ *           //   \\         //  //             //  //   //  //  //
+ *          //     \\  =====//  //        =====//  //   //      //  Version V
  *
- * This program is free software; you can redistribute it and/or modify
+ * ------------- An Open Source RSerPool Simulation for OMNeT++ -------------
+ *
+ * Copyright (C) 2003-2009 by Thomas Dreibholz
+ *
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
 
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
-
+ *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
- * Contact: dreibh@exp-math.uni-essen.de
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
+ * Contact: dreibh@iem.uni-due.de
  */
 
 #include "config.h"
@@ -80,12 +85,9 @@ static int address2string(const struct sockaddr* address,
 #ifdef HAVE_TEST
       case AF_TEST:
          testaddress = (struct sockaddr_testaddr*)address;
-         snprintf(buffer, length, "%u:%u-(x=%1.2f/y=%1.2f/z=%1.2f)",
+         snprintf(buffer, length, "%u:%u",
                   testaddress->ta_addr,
-                  testaddress->ta_port,
-                  testaddress->ta_pos_x,
-                  testaddress->ta_pos_y,
-                  testaddress->ta_pos_z);
+                  testaddress->ta_port);
          return(1);
        break;
 #endif
