@@ -1,4 +1,4 @@
-/* $Id: udplikeserver.cc 1834 2008-07-05 09:31:57Z dreibh $
+/* $Id: udplikeserver.cc 2076 2008-12-28 15:52:34Z dreibh $
  * --------------------------------------------------------------------------
  *
  *              //===//   //=====   //===//   //       //   //===//
@@ -9,7 +9,7 @@
  *
  * ------------- An Efficient RSerPool Prototype Implementation -------------
  *
- * Copyright (C) 2002-2008 by Thomas Dreibholz
+ * Copyright (C) 2002-2009 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -125,7 +125,7 @@ void UDPLikeServer::handleTimer()
 
 
 // ###### Startup ###########################################################
-EventHandlingResult UDPLikeServer::initializeService(int sd)
+EventHandlingResult UDPLikeServer::initializeService(int sd, const uint32_t peIdentifier)
 {
    return(EHR_Okay);
 }
@@ -194,20 +194,20 @@ void UDPLikeServer::poolElement(const char*          programTitle,
          const char* policyName = rsp_getpolicybytype(loadinfo->rli_policy);
          puts("\n\nGeneral Parameters:");
          printf("   Pool Handle             = %s\n", poolHandle);
-         printf("   Reregistration Interval = %u [ms]\n", reregInterval);
+         printf("   Reregistration Interval = %1.3fs\n", reregInterval / 1000.0);
          printf("   Runtime Limit           = ");
          if(runtimeLimit > 0) {
-            printf("%u [s]\n", runtimeLimit);
+            printf("%1.3f [s]\n", runtimeLimit / 1000.0);
          }
          else {
             puts("off");
          }
          puts("   Policy Settings");
          printf("      Policy Type          = %s\n", (policyName != NULL) ? policyName : "?");
-         printf("      Load Degradation     = %1.3f [%%]\n", 100.0 * ((double)loadinfo->rli_load_degradation / (double)PPV_MAX_LOAD_DEGRADATION));
-         printf("      Load DPF             = %1.3f [%%]\n", 100.0 * ((double)loadinfo->rli_load_dpf / (double)PPV_MAX_LOADDPF));
+         printf("      Load Degradation     = %1.3f%%\n", 100.0 * ((double)loadinfo->rli_load_degradation / (double)PPV_MAX_LOAD_DEGRADATION));
+         printf("      Load DPF             = %1.3f%%\n", 100.0 * ((double)loadinfo->rli_load_dpf / (double)PPV_MAX_LOADDPF));
          printf("      Weight               = %u\n", loadinfo->rli_weight);
-         printf("      Weight DPF           = %1.3f [%%]\n", 100.0 * ((double)loadinfo->rli_weight_dpf / (double)PPV_MAX_WEIGHTDPF));
+         printf("      Weight DPF           = %1.3f%%\n", 100.0 * ((double)loadinfo->rli_weight_dpf / (double)PPV_MAX_WEIGHTDPF));
          printParameters();
       }
 
@@ -215,9 +215,9 @@ void UDPLikeServer::poolElement(const char*          programTitle,
       if(rsp_register_tags(RSerPoolSocketDescriptor,
                            (const unsigned char*)poolHandle, strlen(poolHandle),
                            loadinfo, reregInterval, 0, tags) == 0) {
-         if(!quiet) {
-            uint32_t identifier;
-            if(rsp_getsockname(RSerPoolSocketDescriptor, NULL, NULL, &identifier) == 0) {
+         uint32_t identifier = 0;
+         if(rsp_getsockname(RSerPoolSocketDescriptor, NULL, NULL, &identifier) == 0) {
+            if(!quiet) {
                puts("Registration");
                printf("   Identifier              = $%08x\n\n", identifier);
             }
@@ -225,7 +225,7 @@ void UDPLikeServer::poolElement(const char*          programTitle,
          double oldLoad = (unsigned int)rint((double)loadinfo->rli_load / (double)PPV_MAX_LOAD);
 
          // ====== Startup ==================================================
-         const EventHandlingResult initializeResult = initializeService(RSerPoolSocketDescriptor);
+         const EventHandlingResult initializeResult = initializeService(RSerPoolSocketDescriptor, identifier);
          if(initializeResult == EHR_Okay) {
 
             // ====== Main loop =============================================
@@ -321,6 +321,7 @@ void UDPLikeServer::poolElement(const char*          programTitle,
                   }
                }
             }
+            uninstallBreakDetector();
          }
 
          // ====== Shutdown =================================================
